function [z, varargout] = getMdv(p, x1, x2, strf),

% GETMDV return derivative matrix M for the Bhatia-Wolf polynomial. 
%
% GETMDV(p, x1, x2, strf),
%
% matrix where N rows are terms for each (r_i, phi_i) (i = 1 ... N)
% and the columns are for the M terms (j = 1 ... M) 
%
% where 
%       p    = Bhatia-Wolf object
%       x1   = column vector of rs  
%       x2   = row vector of phi
%       strf = 'dzdr','dzdt','u','v'
%
% 13/2/06           -- modified to BHW object
% 26/1/06           -- changed to a switch/case format 
% modified 24/10/05 -- aperture re-scaling added.




nAp = p.aperture;

% added matrix return capability
if (strcmp(p.coord,'cartesian')),
    [phi,r] = cart2pol(x1,x2);
    r       = r/nAp;
else
    r   = x1/nAp;
    phi = x2;
end 
r = r(:); phi = phi(:);

switch(strf),
    case { 'rho','dzdr'}
        z       = bhwRepMtxDv(r, phi, size(p.terms,1), 'rho');
        z       = z*(1/nAp);
        return
    case { 'phi','dzdt' }
        z       = bhwRepMtxDv(r, phi, size(p.terms,1), 'phi');
        return
    case { 'u','v','dzdx','dzdy' }
        dDdr    = bhwRepMtxDv(r, phi, size(p.terms,1), 'rho')*(1/nAp);
        dDdphi  = bhwRepMtxDv(r, phi, size(p.terms,1), 'phi');
        n       = size(dDdr,2);
        phi     = phi(:,ones([ 1 n ] ));
        r       = r(:,ones([ 1 n ] ));
        z       = dpol2dcar(phi, r*nAp, dDdphi, dDdr, strf);
        return
    case { 'u-v' }
        dDdr    = bhwRepMtxDv(r, phi, size(p.terms,1), 'rho')*(1/nAp);
        dDdphi  = bhwRepMtxDv(r, phi, size(p.terms,1), 'phi');
        n       = size(dDdr,2);
        phi     = phi(:,ones([ 1 n ] ));
        r       = r(:,ones([ 1 n ] ));
        z       = dpol2dcar(phi, r*nAp, dDdphi, dDdr, 'u');
        varargout{1} = dpol2dcar(phi, r*nAp, dDdphi, dDdr, 'v');
        return
    
    otherwise 
        error([ 'Unknown flag (' strf ')' ]);
end 
    
return

