function [p, c,e] = fit(p, u, v, f, u0, v0, f0),

% FIT Fit a Zernike object (ZNK) to data (x, y, f)
%
% q = FIT(p, x, y, f).  Fit the Zernike (BHW) object (p) to data 
% in column vectors (x, y, f).  The data should be in the same 
% coordinates as p. If p has been SET to cartesian for example, then data 
% will be expected to be cartesian also. 
% 
% The object returned will be a copy of p, except the coefficients 
% will be those of the polynomial fiited to the data. 
%
% q = FIT(p, x, y, f, u0, v0, f0). A constrained least squares fit.  In 
% this case, the surface is forced to pass through a certain point (u0, v0,
% f0).
%
% Example,
%
%   t = linspace(0, 360, 32); t = t(2:end); t = degtorad(t);
%   r = linspace(0, 1, 32); r = r(2:end);
%   [t, r] = meshgrid(t, r);
%   [x, y] = pol2cart(t, r);
%   z  = sphere(x, y, 7); % sphere with radius = 7
%
%   y0 = znk(7); y0 = set(y0,'coord','cartesian');
%   y0 = fit(y0, x(:), y(:), z(:));
% 

if (nargin < 5),

    % unconstrained least squares
    D   = getM(p, u, v);   
    c   = D\f;
    
else 
    
    % constrained least squares
    D0          = getM(p, u0, v0);
    D           = getM(p, u, v);
    c           = c_lsq(D, f, D0, f0);
    
end

e           = D*c - f;
p.coeff     = c;
% p.residuals = e;

return