 function z = bhwConicCoeffs(CV, CC, rp, nOrder, strf),

% BHWCONICCOEFFS report analytic zernike coefficients for
% the conic with central radial curvature R and conic measure
% k using a 10^th order (66 term) Zernike polynomial.
%
% BHWCONICCOEFFS(CV, CC, rp, str).  Here CV is the conic curvature, 
% CC is the conic constant.  rp is radius of the aperture. The 
% computation uses a least squares based methods. 
%
% BHWCONICCOEFFS(CV, CC, rp, strf).  Same as above except that 
% strf is specified as 'lsq' (least squares) or 'integral' for 
% integration based method.  Both methods should be consistent.
% 
% Example 
%
%   z = bhwConicCoeffs(1/7, 0, 4, 10);

% nOrder    = 10;  %  10 th order Zernike

if (nargin == 0),
    R       = 7.82;   % radius
    K       = 0.82;
    CV      = 1/R; % curvature 
    CC      = K-1;   % conic constant
    rp      = 3.0; % 9 mm diam. aperture
else 
    R = 1/CV;
    K = 1+CC;
end 

if (nargin == 4),
    strf = 'integral';
end 

switch(strf),
    case { 'numeric' },        
        r = linspace(0, 1, 80); r = r(2:end);
        t = linspace(0, 360, 80); t = degtorad(t(1:end-1));
        [r, t] = meshgrid(r, t); r = r(:); t = t(:);
        
        [x0, y0] = pol2cart(t, r); plot(x0, y0, 'x');
        z0 = conic(x0, y0, CV, CC);
 
        n = bhwOrder2Terms(nOrder);
        M = bhwRepMtx(r/rp, t, n);
        z  = M\z0;
        return

    case { 'integral' }
        tol   = 1.e-9;
        f   = [ '((CV*(rp*r)^2)/(1+sqrt(1-(CV)^2*(CC+1)*(rp*r)^2)))' ];
        h  = inline([ f '*r' ], 'r', 'CV', 'CC', 'rp');
        h  = vectorize(h);

        coeff = zeros(bhwOrder2Terms(nOrder), 1);
        coeff(1) = 2*quad(h, 0, 1, tol, [], CV, CC, rp);
        for k=0:1:(nOrder-1),
            nP = bhwOrder2Terms(k)+1;
            g  = [ '(' vectorize(bhwTermStr(nP)) ')' ]; 
            h  = inline([ f '*' g '*r' ], 'r', 'CV', 'CC', 'rp');
            h  = vectorize(h);
            coeff(nP) = 2*quad(h, 0, 1, tol, [], CV, CC, rp);
        end
        z = coeff;
    otherwise,
        error('(BHWCONICCOEFFS) Unknown invocation parameter.');
end 
return