function y = bsp1d(j, p, kt, x)

% BSP1D Compute the height (x) of the basis function for the particular 
% control point
% 
% BSP1D(i, p, kt, x).  Here i is the index of the control point (i = 0,
% ..., N).  The knot sequence is given by kt, p is the degree
%
% A 1D Basis spline is given by, 
%
%               N
%              -----
%      Z(X) =  >  C(I) B(I; P1)(X) 
%              -----
%              I = 0
% 
% and B(I; P1) is the 1-D B-Spline basis function defined by Cox-DeBoor  
% recursive relationship.
% 
% Notes : 
%   
% 05/03/07 modified to be consistent with mathworld definition replaced k (order) 
% with degree (p) :
% 
%   Weisstein, Eric W. "B-Spline." From MathWorld--A Wolfram Web Resource. 
%   http://mathworld.wolfram.com/B-Spline.html 
%

% funny input
m = length(kt) - 1;  % (m+1) knots
if  (j > m-p-1)|(j<0),
    error('bsp1d - inconsistent control point/knot vector relationship.');
end

% init
x  = x(:);
kt = kt(:);

% base thang
if  p == 0,
    % lower is better 
    y       = (kt(j+1)<= x)&(x<kt(j+1+1));
    y       = y(:);
    return
end

a = bsp1dcoeff(j, p, kt);
alpha1 = 0;
if  a(2) ~= 0
    alpha1 = (x - a(1))/a(2);
end

alpha2 = 0;
if  a(4) ~= 0
    alpha2 = (a(3)-x)/a(4);
end

y   = alpha1.*bsp1d(j, p-1, kt, x) + ...
         alpha2.*bsp1d(j+1, p-1, kt, x);

return