function y = znkBasisOSA(r, phi, n, m, repl),

% ZNKBASISOSA zernike basis (OSA) function (L,n)
%
% ZNKBASISOSA(r, phi, n, m, repl). Here r is the radial value, 
% phi is the angular value, (n,m) is the term, and repl is the
% value outside of the aperture (default is 0).
%
%                       { N^m_n R^|m|_n(p) cos(m*theta) m >= 0
%   Z^m_n(p, theta) =  {
%                       { -N^m_n R^|m|_n(p) cos(m*theta) m < 0
%
% where 
%       n           is the radial order index
%       m           is the angular frequency index
%
%       N^m_n       is the normalization factor,
%
%                   sqrt(2*(n+1)/(1+delta)) where delta is delta(m) the
%                   kronecker delta function
%
%       R^|m|_n(p)  is given by,
%
%
%             (n - |m|)/2
%                ---           (-1)^s (n-s)! p^(n-2s)
%   R^|m|_n(p) =  >  -------------------------------------
%                --- s! [0.5*(n+|m|)-s]! [0.5*(n-|m|)-s]!
%                s = 0
% 
% where 
%           r   = distance in unit circle
%           phi = radian angle
%       m   = order
%       n   = order
%       nstr = 'even','odd'
%
% note:
%       0 <= r <= 1
%       0 <= phi < 2*pi
%       n >= m >= 0
% 
% Thibos, LN, Applegate, RA, Schwiegerling, JT, Webb, R. (2000).   
% Standards for reporting the optical Aberrations of Eyes.  
% In: Lakminarayan, V., ed. Trends in Optics and Photonics, Vision Science and Its Applications, 
% OSA Technical Digest Series, Washington, DC: Optical Society of America, Vol. 35:232-244.
% 
% Jim Schwiegerling, John E. Greivenkamp and Joseph M. Miller
% Representation of videokeratoscopic of height data with Zernike
% polynomials. Vol. 12, No. 10/ October 1995/J. Opt. Soc. Am. A

% return 'repl' variable for out-of-bounds co-ordinates 

if (nargin < 5), 
    repl = 0;
end

% return and m=0, n=0 term
if ((m==0)&(n==0))
        y = ones(size(r));  
        y(r>1) = repl;
        return
end


% go to hardcoded
% y = znkQuickBasis(n, m);
% if (~isempty(y)),
%   y = inline(vectorize(y),'r','phi');
%   y = y(r, phi);
%   y(r>1) = repl;
%   return
% end



    delta = inline('n==0');
    N_mn = sqrt((2*(n+1))/(1+delta(m)));
    
    R_mn = 0;
    for s=0:(n-abs(m))/2,
   
        % each zernike term 
        nm      = (-1)^s * factorial(n-s)*r.^(n-2*s);
        
        l = 0.5*(n+abs(m))-s;
        if (isequal(floor(l),l))
            dm      = factorial(s)*factorial(0.5*(n+abs(m))-s)*factorial(0.5*(n-abs(m))-s);
        else 
            warning([ 'fractional factorial! n = ' num2str(n) ', m = ' num2str(m) ]); 
        end
                    
        R_mn    = R_mn + nm/dm;
    
    end
    
    % angular part 
    if  ((m) >= 0),
        y = N_mn.*R_mn.*cos(m*phi);   
    else
        y = -N_mn.*R_mn.*sin(m*phi);   
    end
    y(abs(r)>1) = repl;    
return

