function y = znkTermStr(n, m, q),

% ZNKTERMSTR. Zernike (OSA) term string. 
%
% ZNKTERMSTR(n, m, q) where n is the order, m is the angular frequency
% and q is the numeric precision (optional). 
%
%                       { N^m_n R^|m|_n(p) cos(m*theta) m >= 0
%   Z^m_n(p, theta) =  {
%                       { -N^m_n R^|m|_n(p) cos(m*theta) m < 0
%
% where 
%       n           is the radial order index
%       m           is the angular frequency index
%
%       N^m_n       is the normalization factor,
%
%                   sqrt(2*(n+1)/(1+delta)) where delta is delta(m) the
%                   kronecker delta function
%
%       R^|m|_n(p)  is given by,
%
%
%             (n - |m|)/2
%                ---           (-1)^s (n-s)! p^(n-2s)
%   R^|m|_n(p) =  >  -------------------------------------
%                --- s! [0.5*(n+|m|-s]! [0.5*(n-|m|-s]!
%                s = 0
% 
% where 
%       r   = distance in unit circle
%       phi = radian angle
%       m   = order
%       n   = order
%       nstr = 'even','odd'
%
% note:
%
%   1// Improvement, USE printf with '.0f' option to display large integers
%
%   2//
%       0 <= r <= 1
%       0 <= phi < 2*pi
%       n >= m >= 0
% 
% Thibos, LN, Applegate, RA, Schwiegerling, JT, Webb, R. (2000).   
% Standards for reporting the optical Aberrations of Eyes.  
% In: Lakminarayan, V., ed. Trends in Optics and Photonics, Vision Science and Its Applications, 
% OSA Technical Digest Series, Washington, DC: Optical Society of America, Vol. 35:232-244.
% 
% Jim Schwiegerling, John E. Greivenkamp and Joseph M. Miller
% Representation of videokeratoscopic of height data with Zernike
% polynomials. Vol. 12, No. 10/ October 1995/J. Opt. Soc. Am. A
%
%
%
%



if (nargin < 3),
    q = 10;
end

if (m == 0)&(n==0)
    y = '1';
    return
end 
    
    R_mn = 0;
    p    = (n-abs(m))/2;
    
    % p
 
    R_mn_str = '';
    
    for s=0:p,
           
        
        % each zernike term 
        nm      = (-1)^s * factorial(n-s);
        l       = 0.5*(n+abs(m))-s;
        if (isequal(floor(l),l))
            dm      = factorial(s)*factorial(0.5*(n+abs(m))-s)*factorial(0.5*(n-abs(m))-s);
        else        
            warning([ 'fractional factorial! n = ' num2str(n) ', m = ' num2str(m) ]); 
        end
                    
        if (s > 0),
            r_coeff = signed(nm/dm);
        else 
            r_coeff = num2str(nm/dm);
        end
        
        if (nm/dm ~= 1), 
                    
            if ((n-2*s) ~= 0),
                
                if ((n-2*s) ~= 1),
                    R_mn_str   = [ R_mn_str r_coeff '*r^' num2str(n-2*s) ];    
                else
                    R_mn_str   = [ R_mn_str r_coeff '*r'];    
                end
                    
            else
                R_mn_str   = [ R_mn_str r_coeff ];
            end
            
        else 
        
            if ((n-2*s) ~= 0),
            
                if (n-2*s) ~= 1,
                    R_mn_str   = [ R_mn_str 'r^' num2str(n-2*s) ];
                else
                    R_mn_str   = [ R_mn_str 'r' ];                   
                end
                
            else
            
                if ( s < p),
                    R_mn_str   = [ R_mn_str ];
                else
                    R_mn_str   = [ R_mn_str signed(1) ];
                end
            end
        end
        
        
    end
    
    % 
    delta = inline('n==0');
    
    lmn = (2*(n+1))/(1+delta(m));
    
    if (~iswhole(sqrt(lmn))),  
       N_mn_str = [ 'sqrt(' num2str(lmn,q) ')' ];
    else
       N_mn_str = num2str(sqrt(lmn),q);    
    end  
    
    % angular part 
    if  ((m) >= 0),
        
        if (m>0),
            
            if (m ~= 1),
                y = [ N_mn_str '*(' R_mn_str ')*cos(' num2str(m) '*phi)' ];
            else
                y = [ N_mn_str '*(' R_mn_str ')*cos(phi)' ];              
            end
        else
            y = [ N_mn_str '*(' R_mn_str ')' ];
        end
        
    else
        
        % N_mn        
        if (m ~= -1),
            y = [ asign(-sign(m)) N_mn_str '*(' R_mn_str ')*sin(' num2str(abs(m)) '*phi)' ];   
        else
            y = [ asign(-sign(m)) N_mn_str '*(' R_mn_str ')*sin(phi)' ];          
        end
   end
        
return

function y = iswhole(x),
    y = (floor(x) == x);
return

function y = asign(x),
    
    if (x<0),
       y = '-';
   else 
       y = '';
   end

return

function y = signed(x),
    
    if (x<0),
        y = [ '-' num2str(abs(x)) ];
    elseif (x > 0),
        y = [ '+' num2str(x) ];
    else 
        y = '';
    end

return