function [z, varargout] = toric(x, y, Rx, Ry, str),

% TORIC return a toric surface given explicitly by 
% 
%   z = Rx - sqrt((Rx - Ry + sqrt(Ry^2-y^2))^2 - x^2)
% 
%  where z will be a depth z > 0.  This is the equation described in
% "Field Guide to Visual and Ophthalmic Optics (SPIE Vol. FG04)"  (Schwiegerling). 
%
% TORIC(x, y, Rx, Ry, str). The pair (x, y) is the position in the (x, y) plane, 
% (Rx, Ry) are the radii. of curvatures for the toric.  The optional parameter str 
% is { 'z', 'dzdx', 'dzdy','normal' }.  If strf is omitted, the height ('z') will be 
% returned. 
%
% [Rx, Ry, nArgList ] = TORIC('default').  Returns information for the RAYTRACE 
% toolbox, namely a default conic surface, along with the argument list.
%
% f = TORIC('inline').  Returns an inline version for the depth function ('z')   .
%
% Examples,
%
%   nArgs = toric('default');
%   z = toric(1, 1.5, nArgs{:})
%   z = toric('inline'); z(1, 1.5, nArgs{:})
% 

% 24/10/04  origin centered added. 
% 27/10/04  added compensation when the denominator is zero and x or y is zero. 

if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
            z = { 7, 7.5 };  % default parameters
            varargout = { { 'Rx', 'Ry' } };
        case { 'inline' },
            z = 'Rx - sqrt((Rx - Ry + sqrt(Ry^2-y.^2)).^2 - x.^2) - (Rx - sqrt((Rx - Ry + sqrt(Ry^2))^2))';
            z = inline(z,'x','y','Rx','Ry');
        otherwise,
            error('(TORIC) Unknown invocation parameter.');
    end 
    return
end



% fast height
if (nargin == 4),
    if (Rx == Ry)
        z = sphere(x,y,Rx);
        return
    end
    z  = Rx - sqrt((Rx - Ry + sqrt(Ry^2-y.^2)).^2 - x.^2);
    z0 = Rx - sqrt((Rx - Ry + sqrt(Ry^2))^2);
    z = z - z0;   
    return
end 


switch(str)
  case { 'z' }
    % ----------------------------------------------------------------------  
    % solve this equation for z:
    % 
    %   2    2            2                 2    2              2 1/2     2
    %  y  + x  + (-z - Rx)  + 2 (Rx - Ry) (x  + z  + 2 z Rx + Rx )    - Ry
    %
    %               2
    %    + (Rx - Ry)  = 0
    % 
    % 
    % where z will be a depth where z<0
    % 
    % 
    %                       2                 2    2    2        1/2
    %          z = -Rx + (Rx  - 2 Rx Ry + 2 Ry  - y  - x  - 2 %1)   
    % 
    %                                    2     2    2  1/2
    %          where    %1 := (-(Rx - Ry)  (-Ry  + y ))
    % ----------------------------------------------------------------------  
    % 
    % z   = Rx -(Rx^2-2*Rx*Ry+2*Ry^2-y.^2-x.^2-2*(-(Rx-Ry)^2*(-Ry^2+y.^2)).^(1/2)).^(1/2);
   
    if (Rx == Ry)
        z = sphere(x,y,Rx);
        return
    end
    
    % origin centered
    z  = Rx - sqrt((Rx - Ry + sqrt(Ry^2-y.^2)).^2 - x.^2);
    z0 = Rx - sqrt((Rx - Ry + sqrt(Ry^2))^2);
    z = z - z0;   
    return

  case { 'dzdx' }

    % ---------------------------------------------
    % old version
    % 
    % g   = sqrt(a - b + (b^2 - y.^2).^2 - x.^2); 
    % z  =  (-x)./g;
    % ---------------------------------------------

    % --------------------------------------------------------------------------   
    % dzdx - 
    %        2                 2    2    2                2     2    2  1/2 1/2
    % - x/(Rx  - 2 Rx Ry + 2 Ry  - y  - x  - 2 (-(Rx - Ry)  (-Ry  + y ))   )
    % --------------------------------------------------------------------------  
    
    % j   = find(x ~= 0);
    % z   = zeros([size(x,1),1]);
    % dnm = (Rx^2-2*Rx*Ry+2*Ry^2-y(j,:).^2-x(j,:).^2-2*(-(Rx-Ry)^2*(-Ry^2+y(j,:).^2)).^(1/2)).^(1/2);
    % z0 = -x(j,:)./dnm;
    % z(j,:) = z0;
    
    Zn  = ((Rx-Ry+sqrt(Ry^2-y.^2)).^2 - x.^2);
    dm  = sqrt(Zn);
    
    if (any(dm) == 0)
        warning('toric demoniminator zero [dzdx]');
    end 
    Zp  = x./dm;
    z   = Zp; 
    return
  case { 'dzdy' }
  
    % ---------------------------------------------
    % old verison
    % 
    % g   = sqrt(a - b + (b^2 - y.^2).^2 - x.^2);     
    % z  =  (-2*y*b^2 + 2*y.^3)./g;
    % input('toric wait.');
    % ---------------------------------------------
  
    % -------------------------------------------------------------------------------
    % 
    % dzdy=
    %        /                          2          \
    %        |                 (Rx - Ry)  y        |   /
    %    1/2 |-2 y + 2 ----------------------------|  /
    %        |                    2     2    2  1/2| /
    %        \         (-(Rx - Ry)  (-Ry  + y ))   /
    %
    %             2                 2    2    2                2     2    2  1/2 1/2
    %          (Rx  - 2 Rx Ry + 2 Ry  - y  - x  - 2 (-(Rx - Ry)  (-Ry  + y ))   )
    % ---------------------------------------------------------------------------------
    
    % j  = find(y ~= 0);
    % z  = zeros([size(y,1),1]);    
    % z0 =1/2./(Rx^2-2*Rx*Ry+2*Ry^2-y(j,:).^2-x(j,:).^2-2*(-(Rx-Ry)^2*(-Ry^2+y(j,:).^2)).^(1/2)).^(1/2).*(-2*y(j,:)+2./(-(Rx-Ry)^2.*(-Ry^2+y(j,:).^2)).^(1/2).*(Rx-Ry)^2.*y(j,:)); 
    % z(j,:) = z0;
    
    Zn      = ((Rx-Ry+sqrt(Ry^2-y.^2)).^2 - x.^2);  
    dm      = 2*sqrt(Zn);
    
      
    if (any(dm) == 0)
        warning('toric demoniminator zero [dzdy]');
    end 
     
    dZdy    = -2*(Rx - Ry + sqrt(Ry^2-y.^2)).*(y./sqrt(Ry^2-y.^2));
    Zp      = - dZdy./dm;
    z       = Zp; 
    return
  case { 'normal' }
    z(:,1) = toric(x,y,Rx,Ry,'dzdx');
    z(:,2) = toric(x,y,Rx,Ry,'dzdy');
    z(:,3) = -1;   
    return 
  otherwise
     error('(TORIC) Unknown invocation.');
end


return
