function [z, varargout] = zernike(x, y, nAp, nCoeff, strf),

% ZERNIKE Sum of Zernike polynomial terms (wrapper to the ZNK object)
%
% ZERNIKE(x, y, nAp, nCoeff, nstr). Where (x,y) is the cartesian point, nAp is the 
% aperture, nCoeff is the coefficient vector and strf is either the height ('z'), or 
% the required derivative 'dzdx', 'dzdy', 'dzdr', 'dzdt'. If strf is
% omitted then the default is height.
% 
% [R, nArgList ] = ZERNIKE('default').  Returns information for the RAYTRACE 
% toolbox, namely a default zernike surface, along with the argument list.
%
% f = ZERNIKE('inline').  Not implemented here.
%
% Examples,
%   
%   nArgs = zernike('default'); 
%   z = zernike(1, 1.5, nArgs{:})
%
% 
% see papers by Schwiegerling/Iskander et al. for more information.


if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
            nAp = 3; R = 7; nOrder = 10;
            nCoeff = znkConicCoeffs(1/R, 0, nAp, nOrder); 
            z = { nAp,  nCoeff }; 
            varargout = { { 'AP', 'COEFF' } };
        case { 'inline' },
            z = [];
        otherwise,
            error('(ZERNIKE) Unknown invocation parameter.');
    end
    return
end



% kludgy until fixed.
[n,m] = znkSgl2Dbl(size(nCoeff,1)-1);
p = znk(n); 
p = set(p, 'aperture', nAp);
p = set(p, 'coeff', nCoeff);
p = set(p, 'coord', 'cartesian');

if (nargin == 4),
    z = get(p, x, y);  
    return
end

switch(strf)
  case { 'z' }
    z = get(p, x, y);  
  case { 'dzdx', 'dzdy', 'dzdr', 'dzdt' }
    z = getdv(p, x, y, strf);
  case { 'normal' }
    z(:,1) = getdv(p, x, y, 'dzdx');
    z(:,2) = getdv(p, x, y, 'dzdy'); 
    z(:,3) = -1;   
  otherwise,  
    error('(ZERNIKE) Unknown inovcation.');
end

return
